@@ -19,10 +19,13 @@ class Instrument:
 	step = 0
 	midi_set = False
 	last_tick = (-1, -1) # iteration, tick
+	no_fluidsynth = True
 	
 	def __init__(self, parameters):
-		self.track = Track()
 		self.params = parameters
+		self.track = Track()
+		i = MidiInstrument()
+		self.track.instrument = i
 		if 'start' in parameters:
 			self.start = self.params["start"]
 			if 'end' not in parameters:
@@ -42,38 +45,28 @@ class Instrument:
 
 	def reset(self):
 		self.track = Track()
+		i = MidiInstrument()
+		self.track.instrument = i
+		self.midi_set = False
 		if hasattr(self, "bar"):
 			del self.bar
 
 	def import_fluidsynth(self):
 		global fluidsynth
 		from mingus.midi import fluidsynth
+		self.no_fluidsynth = False
 
 	def add_rest(self, state):
 		if state["tick"] == 0:
-			if hasattr(self, "bar"):
-				self.track + self.bar
-			b = Bar()
-			b.set_meter(state["meter"])
-			b.length = 0.0
-			b.key = state["key"]
-			self.bar = b
-		
+			self.record_new_bar(state)
+
 		dur = state["resolution"]
 		if state["swing"]:
 			if state["tick"] % 2 == 0:
 				dur = 1.0 / (1.0 / dur * (2.0 / 3.0))
 			else:
 				dur = 1.0 / (1.0 / dur * (4.0 / 3.0))
-		if len(self.bar) > 0:
-			if self.bar[0][2] is None:
-				i = 1 / (1.0 / dur + \
-					1.0 / self.bar[0][1])
-				self.bar[0][1] = i
-			else:
-				self.bar.place_notes(None, dur)
-		else:
-			self.bar.place_notes(None, dur)
+		self.record_rest(dur)
 		self.last_tick = (state["iterations"], state["tick"])
 
 
@@ -94,11 +87,10 @@ class Instrument:
 			if not self.no_fluidsynth:
 				fluidsynth.set_instrument(self.params["channel"], self.params["midi_instr"])
 			self.midi_set = True
-			i = MidiInstrument()
-			i.instrument_nr = self.params["midi_instr"]
-			self.track.instrument = i
+			self.track.instrument_nr = self.params["midi_instr"]
 
 	def play(self, state):
+
 		if not(self.midi_set):
 			self.set_instrument()
 
@@ -142,6 +134,8 @@ class Instrument:
 	def record_new_bar(self, state):
 		if hasattr(self, "bar"):
 			self.track + self.bar
+			print self.track
+			print
 		b = Bar()
 		b.set_meter(state["meter"])
 		b.length = 0.0
@@ -155,22 +149,21 @@ class Instrument:
 			self.bar.place_notes(notes, duration)	
 		else:
 			self.bar.bar[-1][2].add_notes(notes)
-			print "sdf?"
 
 	def record_rest(self, duration):
 		"""Adds a rest to the track, or adds a tick to the last\
 NoteContainer if there are notes playing."""
 		dur = duration
 		if len(self.bar) > 0:
-			if self.bar[-1][1] is None or len(self.playing) > 0:
+			if self.bar[-1][2] is None or len(self.playing) > 0:
 				i = 1.0 / (1.0 / dur + 1.0 / self.bar[-1][1])
 				self.bar[-1][1] = i
+				self.bar.current_beat += 1.0 / dur
 			else:
 				self.bar.place_notes(None, dur)
 		else:
 			self.bar.place_notes(None, dur)
 
-
 	def generate_note(self, state):
 		return None
 
